package com.openxware.rss;

import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;

import com.openxware.rss.R;

import android.app.Activity;
import android.content.Context;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AbsListView;
import android.widget.AbsListView.OnScrollListener;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.TextView;

public class LazyAdapter extends BaseAdapter implements OnScrollListener {
    
	private int lastViewedPosition = 0;
	
	private XmlPullParserFactory factory = null;
    private XmlPullParser xpp = null;
    
    private Activity activity;
    private ArrayList<Property> data;
    
    private static LayoutInflater inflater = null;
    public ImageLoader imageLoader;
	private String feedString;
    
    public LazyAdapter(Activity a, String feedString, ArrayList<Property> d) {
        this.feedString = feedString;
        activity = a;
        data = d;
        inflater = (LayoutInflater) activity.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        imageLoader = new ImageLoader(activity.getApplicationContext());
    }

    public int getCount() {
        return data.size();
    }

    public Object getItem(int position) {
        return position;
    }

    public long getItemId(int position) {
        return position;
    }
    
    public static class ViewHolder{
        public TextView text;
        public ImageView image;
    }
    
    public View getView(int position, View convertView, ViewGroup parent) {
        View vi = convertView;
        ViewHolder holder;
        
        if (convertView == null) {
            vi = inflater.inflate(R.layout.item, null);
            holder = new ViewHolder();
            holder.text = (TextView) vi.findViewById(R.id.text);;
            holder.image = (ImageView) vi.findViewById(R.id.image);
            vi.setTag(holder);
        } else {
            holder = (ViewHolder) vi.getTag();
        }
        
        if (data.size() <= position) {
        	readRss(10);
        }
        
        lastViewedPosition = position;
        
        holder.text.setText(data.get(position).getTitle());
        holder.image.setTag(data.get(position).getImage());
        imageLoader.DisplayImage(data.get(position).getImage(), activity, holder.image);
        
        return vi;
    }
    
    private boolean connect() {
	    try {
	    	if (xpp == null) {
				URL url = new URL(feedString);
				URLConnection connection = url.openConnection();
				
				HttpURLConnection httpConnection = (HttpURLConnection)connection; 
				int responseCode = httpConnection.getResponseCode(); 
				
				if (responseCode == HttpURLConnection.HTTP_OK) {
					InputStream in = httpConnection.getInputStream();
					
			    	factory = XmlPullParserFactory.newInstance();
			        factory.setNamespaceAware(true);
			        xpp = factory.newPullParser();
			        
			        xpp.setInput(in, "ISO-8859-1");
				}
	    	}
			
			return true;
			
	    } catch (XmlPullParserException xe) {
	    	Log.e("RssReader", "Connection", xe);
	    	return false;
	    	
	    } catch (IOException eio) {
	    	Log.e("RssReader", "Connection", eio);
	    	return false;
	    	
	    }
    }
    
    private void readRss(int itemsToRead) {
    	boolean insideItem = false;
    	boolean insideTitle = false;
    	boolean insidePubDate = false;
    	boolean insideDescription = false;
    	Property property = null;
    	
    	try {
	    	if (connect()) {
		        int eventType = xpp.getEventType();
		        
		        while (eventType != XmlPullParser.END_DOCUMENT && itemsToRead >= 0) {
		        	if(eventType == XmlPullParser.START_TAG) {
						if (xpp.getName().equalsIgnoreCase("item")) {
							insideItem = true;
							property = new Property();
						} else if (insideItem && xpp.getName().equalsIgnoreCase("title")) {
							insideTitle = true;
						} else if (insideItem && xpp.getName().equalsIgnoreCase("pubDate")) {
							insidePubDate = true;
						} else if (insideItem && xpp.getName().equalsIgnoreCase("description")) {
							insideDescription = true;
						}
						
					} else if(eventType == XmlPullParser.END_TAG) {
						if (xpp.getName().equalsIgnoreCase("item")) {
							insideItem = false;
							data.add(property);
							--itemsToRead;
						}
					    
					} else if(eventType == XmlPullParser.TEXT && insideItem) {
						if (xpp.getText() != null) {
							if (insideTitle) {
								property.setTitle(xpp.getText());
								insideTitle = false;
							} else if (insidePubDate) {
								property.setDate(xpp.getText());
								insidePubDate = false;
							} else if (insideDescription) {
								getDetails(property, xpp.getText());
								insideDescription = false;
							}
						}
						
					}
					
					eventType = xpp.next();
		        }
	    	}
	    	
		} catch (XmlPullParserException e) {
	    	Log.e("RssReader", "Connection", e);
		} catch (IOException e) {
	    	Log.e("RssReader", "Connection", e);
		}
    }
    
	private void getDetails(Property property, String description) {
		int start = description.indexOf("<img src='") + 10;
		int end = description.indexOf("' />");
		String imageURL = description.substring(start, end);
		property.setImage(imageURL);
		
		String desc = description.replaceAll("\\<.*?\\>", "");
		String[] parts = desc.split("\n");
		
		for (String line : parts) {
			for (Filter filter : Filter.values()) {
				if ((line.toUpperCase()).startsWith(filter.name())) {
					switch (filter) {
						case CATEGORY:
							property.setCategory((line.substring(filter.name().length() + 1)).trim());
							break;
							
						case BEDROOMS:
							property.setBedrooms(Integer.valueOf((line.substring(filter.name().length() + 1)).trim()));
							break;
							
						case BATHROOMS:
							property.setBathrooms(Integer.valueOf((line.substring(filter.name().length() + 1)).trim()));
							break;
					}
				}
			}
		}
	}

	@Override
	public void onScroll(AbsListView view, int firstVisibleItem, int visibleItemCount, int totalItemCount) {
		if (totalItemCount - lastViewedPosition <= 3) {
			readRss(10);
		}
	}

	@Override
	public void onScrollStateChanged(AbsListView view, int scrollState) {
		// TODO Auto-generated method stub
    	Log.d("RssReader", "onScrollStateChanged");
	}

}